@@ -14,6 +14,7 @@ use Data::Dumper 'Dumper';
 use Digest::MD5 'md5_hex';
 use Carp 'croak';
 use CGI 'pre';
+use Storable 'lock_store','lock_retrieve';
 
 my %CONFIG_CACHE; # cache parsed config files
 my %DB_SETTINGS;  # cache database settings
@@ -73,20 +74,16 @@ sub new {
   $self->config_file($config_file_path);
   $self->add_scale_tracks();
   $self->precompile_glyphs;
+
+  # this generates the invert_types data structure which will then get cached to disk
+  $self->type2label('foo',0,'bar');
+
   $CONFIG_CACHE{$config_file_path}{object} = $self;
   $CONFIG_CACHE{$config_file_path}{mtime}  = $mtime;
   $CONFIG_CACHE{$config_file_path}{ctime}  = time();
   return $self;
 }
 
-sub _new {
-    my $class = shift;
-    my $self  = $class->SUPER::_new(@_);
-    # this generates the invert_types data structure which will then get cached to disk
-    $self->_type2label($self,'foo','bar');
-    return $self;
-}
-
 sub name {
   my $self = shift;
   my $d    = $self->{name};
@@ -287,7 +284,7 @@ sub details_multiplier {
 
 sub unit_label {
   my $self  = shift;
-  my $value = shift;
+  my $value = shift || 0;
 
   my $unit     = $self->setting('units')        || 'bp';
   my $divider  = $self->setting('unit_divider') || 1;
@@ -361,7 +358,6 @@ sub remote_renderer {
     return $self->global_setting('remote renderer');
 }
 
-
 sub labels {
   my $self   = shift;
 
@@ -409,6 +405,15 @@ sub label2type {
   return shellwords($self->setting($l,'feature')||$self->setting($label,'feature')||'');
 }
 
+sub track_listing_class {
+    my $self = shift;
+    my $style    =  lc $self->global_setting('track listing style') || 'categories';
+    my $subclass =   $style eq 'categories' ? 'Categories'
+	           : $style eq 'facets'     ? 'Facets'
+		   : die "invalid track listing style '$style'";
+    return 'Bio::Graphics::Browser2::Render::HTML::TrackListing::'.$subclass;
+}
+
 sub seqid_prefix { shift->fallback_setting(general=>'seqid_prefix') }
 
 sub default_style {
@@ -681,87 +686,87 @@ sub type2label {
   $dbid   ||= '';
   $type   ||= '';
   $length ||= 0;
+  $type = lc $type;
+  $dbid =~ s/:database//;
 
-  my @labels;
-
-  if (exists $self->{_type2labelmemo}{$type,$dbid}) {
-      @labels =  @{$self->{_type2labelmemo}{$type,$dbid}};
+  if (exists $self->{_type2label}{$type,$length,$dbid}) {
+      my @labels = @{$self->{_type2label}{$type,$length,$dbid}};
+      return wantarray ? @labels : $labels[0];
   }
 
-  else {
-      my @main_labels = $self->_type2label($self,
-					   $type,
-					   $dbid);
-      my @user_labels = $self->_type2label($self->{_user_tracks},
-					   $type,
-					   $dbid);
-      my %label_groups;
-      for my $label (@main_labels,@user_labels) {
-	  my ($label_base,$minlength) = $label =~ /(.+)(?::(\d+))?/;
-	  $minlength ||= 0;
-	  next if defined $length && $minlength > $length;
-	  $label_groups{$label_base}++;
-      }
-      @labels = keys %label_groups;
-      $self->{_type2labelmemo}{$type,$dbid} = \@labels;
-  }
+  my %labels;
+  my $main_inverted = $self->invert_types($self->{config});
+  my $user_inverted = $self->invert_types($self->{_user_tracks}{config});
 
+  for my $i ($main_inverted,$user_inverted) {
+      my @lengths = grep {$length >= $_} keys %{$i->{$type}{$dbid}};
+      $labels{$_}++ foreach map {keys %{$i->{$type}{$dbid}{$_}}} @lengths;
+  }
+  my @labels = keys %labels;
+  $self->{_type2label}{$type,$length,$dbid} = \@labels; #cache in memory
   return wantarray ? @labels : $labels[0];
 }
 
-sub _type2label {
-    my $self = shift;
-    my ($storage_hash,$type,$dbid) = @_;
-
-    my $type2label = $storage_hash->{_type2label}{$storage_hash}
-                 ||= $self->invert_types($storage_hash->{config});
-    $dbid =~ s/:database$//;
-    my @labels = keys %{$type2label->{lc $type}{$dbid}};
-    return wantarray ? @labels : $labels[0];
-}
-
 # override inherited in order to allow for semantic zooming
 sub feature2label {
   my $self = shift;
   my ($feature,$length) = @_;
   my $type  = eval {$feature->type}
     || eval{$feature->source_tag} || eval{$feature->primary_tag} or return;
-
   my $dbid = eval{$feature->gbrowse_dbid};
   
   (my $basetype = $type) =~ s/:.+$//;
-  my @label = $self->type2label($type,$length,$dbid);
-  push @label,$self->type2label($basetype,$length,$dbid);
+  my %label;
 
-  # remove duplicate labels
-  my %seen;
-  @label = grep {! $seen{$_}++ } @label; 
+  $label{$_}++ foreach $self->type2label($type,$length,$dbid);
+  $label{$_}++ foreach $self->type2label($basetype,$length,$dbid);
 
+  my @label = keys %label;
   wantarray ? @label : $label[0];
 }
 
 sub invert_types {
-  my $self    = shift;
-  my $config  = shift;
-  return unless $config;
+    my $self   = shift;
+    my $config = shift;
+    return unless $config;
 
-  my %inverted;
-  for my $label (sort keys %{$config}) {
-      my $length = 0;
-      if ($label =~ /^(.+):(\d+)$/) {
-       	  $label  = $1;
-       	  $length = $2;
-      }
+    my $keys         = md5_hex(keys %$config);
 
-      $length *= $self->global_setting('details multiplier') || 1;
-      my $feature = $self->semantic_setting($label => 'feature',$length) or next;
-      my ($dbid)  = $self->db_settings($label => $length) or next;
-      $dbid =~ s/:database$//;
-      foreach (shellwords($feature||'')) {
-	  $inverted{lc $_}{$dbid}{$label}++;
-      }
-  }
-  \%inverted;
+    # check in-memory cache
+    if (exists $self->{_inverted}{$keys}) {
+	return $self->{_inverted}{$keys};
+    }
+
+    # check for a valid cache file
+    (my $cache_name     = $self->config_file) =~ s!/!_!g;
+    my $master_cache = $self->cachefile($cache_name);  # inherited
+    $master_cache   or die "no bio::graphics cache file found at $master_cache? Shouldn't happen..";
+
+    my $inv_path     = $master_cache . ".${keys}.inverted";
+    if (-e $inv_path && -M $master_cache >= -M $inv_path) {
+	return $self->{_inverted}{$keys} = lock_retrieve($inv_path);
+    }
+
+    my $multiplier = $self->global_setting('details multiplier') || 1;
+
+    my %inverted;
+    for my $label (sort keys %{$config}) {
+	my $length = 0;
+	if ($label =~ /^(.+):(\d+)$/) {
+	    $label  = $1;
+	    $length = $2;
+	}
+
+	$length    *= $multiplier;
+	my $feature = $self->semantic_setting($label => 'feature',$length) or next;
+	my ($dbid)  = $self->db_settings($label      => $length) or next;
+	$dbid =~ s/:database$//;
+	foreach (shellwords($feature||'')) {
+	    $inverted{lc $_}{$dbid}{$length}{$label}++;
+	}
+    }
+    lock_store(\%inverted,$inv_path);
+    return  $self->{_inverted}{$keys} = \%inverted;
 }
 
 sub default_labels {
